/*
 * wimlib.h
 *
 * External header for wimlib.
 *
 * Copyright (C) 2012 Eric Biggers
 *
 * wimlib - Library for working with WIM files 
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option) any
 * later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

/** \mainpage
 *
 * \section intro Introduction
 *
 * Wimlib is a C library to read, write, and mount archive files in the Windows
 * Imaging Format (WIM files).  These files are normally created using the @c
 * imagex.exe utility on Windows, but this library provides a free
 * implementetion of @c imagex for UNIX-based systems and an API to allow other
 * programs to read, write, and mount WIM files.
 * 
 * The main intended use of wimlib is to create images of Windows PE, the
 * Windows Preinstallation Environment, without having to rely on Windows.
 * Windows PE, which is the operating system that runs when you boot from the
 * Windows Vista or Windows 7 DVD, is a lightweight version of Windows that can
 * run entirely from memory. It can be used to install Windows from local media
 * or a network drive or perform maintenance. 
 * 
 * You can find Windows PE on the installation media for Windows Vista, Windows
 * 7, and Windows 8.  The Windows PE image itself is a WIM file, @c
 * sources/boot.wim, on the ISO filesystem.  Windows PE can also be found in the
 * Windows Automated Installation Kit (WAIK) inside the @c WinPE.cab file, which
 * you can extract if you install the @c cabextract program.
 *
 * \section format WIM files
 *
 * A <b>Windows Imaging (WIM)</b> file is an archive that contains one or
 * more directory trees.  Like some other archive formats such as ZIP, files in
 * WIM archives may be compressed.  WIM archives support two Microsoft-specific
 * compression formats:  @b LZX and @b XPRESS.  A WIM file can use at
 * most one compression format, and need not use compression at all.  Wimlib
 * supports all of these formats, both for reading and writing.
 *
 * Unlike ZIP files, WIM files can contain multiple independent directory trees
 * known as @a images.  While each image needs its own metedata, files are not
 * duplicated for each image; instead, each file is included only once in the
 * entire WIM. Microsoft did this so that in one WIM file, they could do things
 * like have 5 different versions of Windows that are almost exactly the same.
 *
 * Microsoft provides documentation for the WIM file format, XPRESS compression
 * format, and LZX compression format.  However, there are errors and omissions
 * in some places in their documentation.
 *
 * \section starting Getting Started
 *
 * Wimlib uses the GNU autotools, so it should be easy to install with
 * <code>configure && make && sudo make install</code>, provided that you have
 * @c libxml2 and @c libfuse installed.  To use wimlib in a program after
 * installing it, include @c wimlib.h and link your program with @c -lwim.
 *
 * Wimlib wraps up a WIM file in an opaque @c WIMStruct structure.
 *
 * All functions in Wimlib's public API are prefixed with @c wimlib, and most
 * return the boolean @c true on success and @c false on failure.  If a library
 * function fails, usually an error message is automatically printed; to
 * suppress this, use wimlib_set_print_errors().  
 *
 * Wimlib is currently almost entirely thread-safe as long as different
 * <code>WIMStruct</code>s are used.  However, wimlib_set_print_errors() applies
 * globally, and wimlib_mount() can only be used by one @c WIMStruct at a time.
 *
 * To open an existing WIM, use wimlib_open_wim(), which begins reading the
 * specified WIM file and returns a pointer to a @c WIMStruct, on which further
 * operations can be performed.
 *
 * To create a new WIM, use wimlib_create_new_wim(), which creates a new @c
 * WIMStruct that contains no images.
 *
 * To add an image to a WIM file from a directory tree on your filesystem, use
 * wimlib_add_image().
 *
 * To extract an image from a WIM file, use wimlib_set_output_dir() to set the
* output directory, then use wimlib_extract_image().
 *
 * Wimlib supports mounting WIM files either read-only or read-write.  Mounting
 * is done using wimlib_mount() and unmounting is done using wimlib_unmount().
 * Mounting can be done without root privileges because it is implemented using
 * FUSE (Filesystem in Userspace).  If Wimlib is compiled with the
 * <code>--without-fuse</code> flag, these functions will be available but will
 * fail.
 *
 * After creating or modifying a WIM file, you can write it to a file using
 * wimlib_write().  Alternatively,  if the WIM was originally read from a file,
 * you can use wimlib_overwrite() to overwrite the original file.
 *
 * After you are done with your WIM file, use wimlib_free() to free all memory
 * associated with a @c WIMStruct and close all files associated with it.
 *
 * To see an example of how to use wimlib, see the file
 * @c programs/imagex.c in wimlib's source tree.
 *
 * \section imagex imagex
 *
 * Wimlib comes with the <b>imagex</b> program, which is documented in man pages.
 *
 * \section mkwinpeimg mkwinpeimg
 * 
 * Wimlib comes with the <b>mkwinpeimg</b> script, which is documented in a man
 * page.
 *
 * \section Limitations
 *
 * While Wimlib supports the main features of WIM files, Wimlib currently has
 * the following limitations:
 * - There is no support for split WIMs, which that have an image divided up
 *   between multiple WIM files (presumably in order to fit them on multiple
 *   CD's or something)
 * - There is not yet any code to verify that there are no collisions between
 *   different files that happen to have the same SHA1 message digest.
 *   This is extremely unlikely, but could result in something bad such as a
 *   file going missing.
 * - Wimlib currently does nothing with the security data in WIM files other
 *   than reading it in, and possibly printing some unhelpful information about
 *   it.  File permissions are ignored.  However, Windows PE is still able to
 *   boot using WIM created by Wimlib without any security data.
 * - Alternate stream entries for directory entries are read, but are ignored.
 * - There is no code to deal with different versions of the WIM file format, if
 *   they even exist.
 * - Chunk sizes other than 32768 are unsupported.  However, as far as I can
 *   tell, other chunk sizes are not used in WIM files.
 * - The use of custom memory allocators is not yet allowed
 * - There is not yet a function to directly change the compression type of a
 *   WIM file; however, this limitation can be worked around by creating a new
 *   WIM with the desired compression type using wimlib_create_new_wim(), then
 *   exporting all the images to the new WIM using wimlib_export_image().
 * - Wimlib does not provide a clone of the @b PEImg tool that allows you to
 *   make certain Windows-specific modifications to a Windows PE image, such as
 *   adding a driver or Windows component.  Such a tool could conceivably be
 *   implemented on top of Wimlib, although it likely would be hard to implement
 *   because it would have to do very Windows-specific things such as
 *   manipulating the driver store.  Wimlib does provide the @b mkwinpeimg
 *   script for a similar purpose, however.  With regards to adding drivers to
 *   Windows PE, you have the option of putting them anywhere in the Windows PE
 *   image, then loading them after boot using @b drvload.exe.
 * - There is not yet a way to extract specific files or directories from a WIM
 *   file without mounting it, or to add, remove, or modify files in a WIM
 *   without mounting it, other than by adding or removing an entire image.  I
 *   can implement this if requested, but I intend the FUSE mount feature to be
 *   used for this purpose, as it is easy to do these things in whatever way you
 *   want after the image is mounted.
 *
 * Wimlib's current error reporting infastructure is nice for console programs,
 * but it isn't much help if you want to know in your code exactly why a
 * function failed.
 *
 * Wimlib currently will print an error message and call @c abort() if a call to
 * @c malloc() returns @c NULL.
 *
 * Currently, Microsoft's @a image.exe can create slightly smaller WIM files
 * than wimlib when using maximum (LZX) compression because it knows how to
 * split up LZX compressed blocks, which is not yet implemented in wimlib.
 *
 * Wimlib is experimental and likely contains bugs; use Microsoft's @a
 * imagex.exe if you want to make sure your WIM files are made correctly.
 *
 * \section legal License
 *
 * The wimlib library is licensed under the GNU Lesser General Public License
 * version 2.1 or later.
 *
 * @b imagex and @b mkwinpeiso are licensed under the GNU General Public License
 * version 3 or later.
 */

#ifndef _WIMLIB_H
#define _WIMLIB_H

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>

#ifndef _WIMLIB_INTERNAL_H
/** 
 * Opaque structure that represents a WIM file. 
 */
typedef struct WIMStruct WIMStruct;
#endif

/** 
 * Specifies the way in which identical files are linked when extracting
 * image(s) from the WIM. 
 */
enum wim_link_type {
/** Hard link identical files when extracting files from the WIM. */
	WIM_LINK_TYPE_HARD = 0,
/** Symbolic link identical files when extracting files from the WIM. */
	WIM_LINK_TYPE_SYMBOLIC = 1,
/** Do not create links when extracting identical files from the WIM (default) */
	WIM_LINK_TYPE_NONE = 2,
};

/**
 * Specifies the compression type of a WIM file.
 */

enum wim_compression_type {
	/** An invalid compression type. */
	WIM_COMPRESSION_TYPE_INVALID = -1,

	/** The WIM does not compress resources. */
	WIM_COMPRESSION_TYPE_NONE = 0,

	/** Compressed resources in the WIM use LZX compression. */
	WIM_COMPRESSION_TYPE_LZX = 2,

	/** Compressed resources in the WIM use XPRESS compression. */
	WIM_COMPRESSION_TYPE_XPRESS = 3,
};


/** Used to indicate that no WIM image is currently selected. */
#define WIM_NO_IMAGE   (-1)

/** Used to specify all images in the WIM. */
#define WIM_ALL_IMAGES (-2)


/**
 * Adds an image to a WIM file from a directory tree on disk.
 *
 * The directory tree is read immediately for the purpose of constructing a
 * directory entry tree in-memory.  Also, all files are read to calculate their
 * SHA1 message digests.  However, because the directory tree may contain a very
 * large amount of data, the files themselves are not read into memory, and
 * instead references to their paths saved.  This means that the directory tree
 * must not be modified before executing a call to wimlib_write() or
 * wimlib_overwrite(). Otherwise, wimlib_write() may fail or incorrect files may
 * be included in the WIM written by wimlib_write().
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param path
 * 	The path to a directory in the outside filesystem.  It will become the
 * 	root directory for the WIM image.
 * @param name
 * 	The name to give the image.  This parameter must be specified.
 * @param desc
 * 	The description to give the image.  This parameter may be left @c
 * 	NULL, in which case no description is given to the image.
 * @param flags
 * 	What to put in the &lt;FLAGS&gt; element for the image's XML data.  This
 * 	parameter may be left @c NULL, in which case no &lt;FLAGS&gt; element is
 * 	given to the image.
 * @param boot
 * 	@c true to change the image marked as bootable to the new one; @c false
 * 	to leave the boot index alone.
 *
 * @return
 * 	@c true on success, @c false for any of the following reasons:
 * 	- @a name was @c NULL.
 * 	- @a name is already used by an image in the WIM.
 * 	- @a path does not specify a readable directory.
 * 	- A file or directory within the directory @a path could not be read.
 */
bool wimlib_add_image(WIMStruct *w, const char *path, const char *name, 
			const char *desc, const char *flags, bool boot);

/** 
 * Creates a WIMStruct for a new WIM file.
 *
 * @param ctype 
 * 	The type of compression to be used in the new WIM file.  Must be
 * 	::WIM_COMPRESSION_TYPE_NONE, ::WIM_COMPRESSION_TYPE_LZX, or
 * 	::WIM_COMPRESSION_TYPE_XPRESS.
 * @return 
 * 	A pointer to a ::WIMStruct for the new WIM file.  The ::WIMStruct must
 * 	be freed using wimlib_free() when finished with it.
 */
WIMStruct *wimlib_create_new_wim(int ctype);

/**
 * Deletes an image, or all images, from a WIM file.
 *
 * All file resources referenced from the image(s) being deleted are removed
 * from the WIM if they are not referenced in any other images in the WIM.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param image
 * 	The number of the image to delete, or ::WIM_ALL_IMAGES to delete all
 * 	images.
 *
 * @return 
 * 	@c true on success; @c false if the metadata for any of the images being
 * 	deleted could not be read, or if @a image was invalid.
 */
bool wimlib_delete_image(WIMStruct *w, int image);

/**
 * Copies an image, or all the images, from a WIM file, into another WIM file.
 *
 * @param src_w
 * 	The ::WIMStruct for the WIM file that contains the image(s) being
 * 	exported.
 * @param src_image
 * 	The image to export from @a src_w.  Can be the number of an image, or
 * 	::WIM_ALL_IMAGES to export all images.
 * @param dest_w
 * 	The ::WIMStruct for the WIM file that will receive the images being
 * 	exported.
 * @param dest_image_name
 * 	The name to give the exported image in the new WIM file.  If left @c NULL,
 * 	the name from the source WIM is used.  This parameter must be left @c NULL
 * 	if @a src_image is ::WIM_ALL_IMAGES and @a src_w contains more than one
 * 	image; in that case, the names are all taken from the source WIM.
 * @param dest_description
 * 	The description to give the exported image in the new WIM file.  If left
 * 	@c NULL, the description from the source WIM is used.  This parameter must
 * 	be left @c NULL if @a src_image is ::WIM_ALL_IMAGES and @a src_w contains
 * 	more than one image; in that case, the descriptions are all taken from
 * 	the source WIM.
 * @param boot
 * 	@c true if the image being exported is to be made bootable, and @c false
 * 	if which image is marked as bootable in the destination WIM is to be
 * 	left unchanged.  If @a src_image is ::WIM_ALL_IMAGES and there are
 * 	multiple images in the source WIM, specifying <code>boot = true</code>
 * 	is valid only if one of the exported images is currently marked as
 * 	bootable in the source WIM; if that is the case, then that image is
 * 	marked as bootable in the destination WIM.
 *
 * @return
 * 	@c true on success, @c false on failure.  This function may fail for any
 * 	of the following reasons:
 * 	- @a boot was @c true, @a src_image was ::WIM_ALL_IMAGES, @a src_w
 * 	contains multiple images, and no images in @a src_w are marked as
 * 	bootable.
 * 	- @a dest_image_name and/or @a dest_description were
 * 	non-<code>NULL</code>, @a src_image was ::WIM_ALL_IMAGES, and @a src_w
 * 	contains multiple images.
 * 	- The metadata for an image being exported could not be read.
 * 	- One or more of the names being given to an exported image was already
 * 	in use in the destination WIM.
 */
bool wimlib_export_image(WIMStruct *src_w, int src_image, 
			WIMStruct *dest_w, const char *dest_image_name, 
			const char *dest_description, bool boot);

/**
 * Extracts an image, or all images, from a WIM file.
 *
 * The output directory must have been previously set with
 * wimlib_set_output_dir().
 *
 * The link type used for extracted files is that specified by a previous call
 * to wimlib_set_link_type(), or @c WIM_LINK_TYPE_NONE by default.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param image
 * 	The image to extract.  Can be the number of an image, or ::WIM_ALL_IMAGES
 * 	to specify that all images are to be extracted.
 *
 * @return
 * 	@c true if all files were successfully extracted; @c false for any of
 * 	the following reasons:
 * 	- There was an error when writing the data to the filesystem or when
 * 	creating a symbolic link or hard link.
 * 	- The metadata for an image being extracted could not be read.
 * 	- There was an error when reading data from the WIM.
 * 	- A compressed file resource in the WIM could not be decompressed, due
 * 	to the compressed data being invalid.
 * 	- No successful call to wimlib_set_output_dir() had previously been made
 * 	on @a w.
 */
bool wimlib_extract_image(WIMStruct *w, int image);

/**
 * Extracts the XML data for a WIM file to a file stream.  Every WIM file
 * includes a string of XML that describes the images contained in the WIM.
 *
 * @param w 
 * 	The ::WIMStruct for the WIM file.
 * @param fp 
 * 	@c stdout, or a FILE* opened for writing, to extract the data to.  
 *
 * @return 
 * 	@c true if the data was successfully extracted; otherwise @c false.
 */
bool wimlib_extract_xml_data(WIMStruct *w, FILE *fp);

/**
 * Frees all memory allocated for a WIMStruct and closes all files associated
 * with it. 
 *
 * @param w
 * 	The ::WIMStruct for a WIM file.
 *
 * This function has no return value.
 */
void wimlib_free(WIMStruct *w);

/**
 * Finds which image in a WIM is bootable.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * 
 * @return
 * 	0 if no image is marked as bootable, or the number of the image marked
 * 	as bootable.
 */
int wimlib_get_boot_idx(const WIMStruct *w);

/**
 * Gets the compression type used in the WIM.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * 
 * @return
 * 	::WIM_COMPRESSION_TYPE_NONE, ::WIM_COMPRESSION_TYPE_LZX, or
 * 	::WIM_COMPRESSION_TYPE_XPRESS.
 */
int wimlib_get_compression_type(const WIMStruct *w);

/**
 * Converts a compression type enumeration value into a string.
 *
 * @param ctype
 * 	::WIM_COMPRESSION_TYPE_NONE, ::WIM_COMPRESSION_TYPE_LZX,
 * 	::WIM_COMPRESSION_TYPE_XPRESS, or another value.
 *
 * @return
 * 	"None", "LZX", "XPRESS", or "Invalid", respectively.
 */
const char *wimlib_get_compression_type_str(int ctype);

/**
 * Returns the description of the specified image.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param image
 * 	The number of the image.
 *
 * @return
 * 	The description of the image, or @c NULL if there is no such image, or @c NULL
 * 	if the specified image has no description.
 */
const char *wimlib_get_image_description(const WIMStruct *w, int image);

/**
 * Returns the name of the specified image.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param image
 * 	The number of the image.
 *
 * @return
 * 	The name of the image, or @c NULL if there is no such image.
 */
const char *wimlib_get_image_name(const WIMStruct *w, int image);


/**
 * Gets the number of images contained in the WIM.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * 
 * @return
 * 	The number of images contained in the WIM file.
 */
int wimlib_get_num_images(const WIMStruct *w);


/**
 * Determines if an image name is already used by some image in the WIM.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param name
 * 	The name to check.
 *
 * @return
 * 	@c true if some other image is already named @a name; @c false if there
 * 	is no existing image named @a name.
 */
bool wimlib_image_name_in_use(const WIMStruct *w, const char *name);

/**
 * Mounts an image in a WIM file on a directory read-only or read-write.
 *
 * A daemon will be forked to service the filesystem.
 *
 * If the mount is read-write, modifications to the WIM are staged in a staging
 * directory.
 *
 * wimlib_mount() currently cannot be used with multiple ::WIMStruct's without
 * intervening wimlib_unmount()s.  If there was a way to have @c libfuse pass a
 * pointer to user data to each FUSE callback, then this would be possible, but
 * there doesn't seem to be a way to do this currently.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param image
 * 	The number of the image to mount.  It must be an existing, single image.
 * @param dir
 * 	The path to an existing directory to mount the image on.
 * @param ro
 * 	@c true if the mount is to be read-only; @c false if the mount is to be
 * 	read-write.
 * @param debug
 * 	This parameter should be set to @c false. (If set to @c true, @c -d is
 * 	added to the FUSE mount options.)
 *
 * @return
 * 	@c true if the mount was successful; @c false on failure.  This function
 * 	can fail for any of the following reasons:
 * 	- @a dir does not exist.
 * 	- @a image was invalid.
 * 	- @c ro was false, and the staging directory could not be created.
 * 	- The current working directory could not be determined.
 * 	- @c fuse_main() returned a failure status.
 *
 */
bool wimlib_mount(WIMStruct *w, int image, const char *dir, bool ro,
						bool debug);

/**
 * Opens a WIM file and creates a ::WIMStruct for it.
 *
 * @param wim_file 
 * 	The path to the WIM file to open.
 * @param check 
 * 	@c true if the integrity information of the WIM, if it exists, is to be
 * 	checked.  The integrity information is a table, located at the end of
 * 	the WIM file, of SHA1 message digests of 10MB chunks of the WIM file.
 * 	If this parameter is @c true and an integrity table exists, these SHA1
 * 	message digests will be verified, and @c NULL will be returned if any of
 * 	the checks fail.  If this parameter is @c false, the integrity table is
 * 	ignored.  If the integrity table does not exist, this parameter has no
 * 	effect.
 *
 * @return 
 * 	An success, a pointer to the ::WIMStruct for the WIM file is returned.  On
 * 	failure, @c NULL is returned.  This function can fail for any of the
 * 	following reasons:
 * 	- @a wim_file could not be opened for reading. 
 * 	- An unexpected stream error or end-of-file occurred while attempting to
 * 	read data from @a wim_file, or while attempting to seek to a certain
 * 	position in @a wim_file.
 * 	- The WIM header was an unexpected size.
 * 	- @a check was specified, the WIM file contained an integrity table, and
 * 	the integrity check failed, either due to the SHA1 message digest of a
 * 	WIM chunk being incorrect, or due to the integrity table being invalid.
 * 	- @a wim_file is compressed, but the chunk size is not 32768.
 * 	- @a wim_file is compressed, but the compression method is not marked as
 * 	LZX or XPRESS.
 * 	- The lookup table of @a wim_file was compressed.
 * 	- Fewer or more metadata resources were found in the WIM than the image
 * 	count given in the WIM header.
 * 	- The number of <IMAGE> elements in the XML data for the WIM did not
 * 	match the image count given in the WIM header.
 */
WIMStruct *wimlib_open_wim(const char *wim_file, bool check);

/**
 * Overwrites the file that the WIM was originally read from, with changes made.
 *
 * The new WIM is created in a temporary file and then renamed to the original
 * file when it is completely written.  The temporary file currently is made in
 * the same directory as the original WIM file.
 *
 * Note that it is not possible for this function to delete the original file
 * before having written the new file because it is very likely that file
 * resources in the new WIM file need to be retrieved from the old WIM file.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.  There may have been in-memory changes
 * 	made to it, which are then reflected in the output file.
 * @param check_integrity 
 * 	@c true if an integrity table is to be included in the WIM being
 * 	written; @c false otherwise.
 *
 * @return
 * 	@c true if the WIM was successfully written.  @c false can be returned
 * 	for any of the following reasons:
 * 	- @a w corresponds to a WIM created with wimlib_create_new_wim() rather
 * 	than a WIM read with wimlib_open_wim().
 * 	- An error occurred when trying to open the temporary file or write data
 * 	to it.
 * 	- The new WIM includes references to files in the outside filesystem, and
 * 	an error occured while trying to read one of these files, perhaps due to
 * 	the file being deleted or changed.
 * 	- The new WIM includes references to file resources in the original WIM,
 * 	but an error occurred while trying to read these resources from the
 * 	original WIM.
 */
bool wimlib_overwrite(WIMStruct *w, bool check_integrity);

/**
 * Updates the header and XML data of the WIM file, without the need to write
 * out the entire WIM to a temporary file as in wimlib_write().
 *
 * This function must only be used if no files, directories, or images have been
 * added, removed, or changed in the WIM.  It must be used when only the boot
 * index or the name or description of image(s) has been changed.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param check_integrity 
 * 	@c true if an integrity table is to be included in the WIM being
 * 	written; false otherwise.
 *
 * @return
 * 	@c true if the WIM was successfully written.  @c false can be returned
 * 	for any of the following reasons:
 * 	- @a w corresponds to a WIM created with wimlib_create_new_wim() rather
 * 	than a WIM read with wimlib_open_wim().
 * 	- An error occurred when trying to open the original WIM file
 * 	read-write.
 * 	- An error occurred when trying to write the data to the original WIM file.
 *
 */
bool wimlib_overwrite_xml_and_header(WIMStruct *w, bool check_integrity);

/**
 * Prints information about an image contained in a WIM file.
 *
 * @param w 
 * 	The ::WIMStruct for the WIM file.
 * @param image 
 * 	The image about which to print information.  Can be the number of an
 * 	image, or ::WIM_ALL_IMAGES to print information about all images in the
 * 	WIM.
 * 
 * This function has no return value.
 *
 */
void wimlib_print_available_images(const WIMStruct *w, int image);

/**
 * Prints the full paths to all files contained in an image in a WIM file.
 *
 * @param w 
 * 	The ::WIMStruct for the WIM file.
 * @param image 
 * 	Which image to print files for.  Can be the number of an image, or
 * 	::WIM_ALL_IMAGES to print the files contained in all images.  
 *
 * @return 
 * 	@c true on success, @c false on failure.  The function may fail due to
 * 	an invalid @a image parameter or due to the inability to read the
 * 	metadata for an image.
 */
bool wimlib_print_files(WIMStruct *w, int image);

/**
 * Prints detailed information from the header of a WIM file.
 *
 * @param w 
 * 	The ::WIMStruct for the WIM file.
 *
 * This function has no return value.
 *
 */
void wimlib_print_header(const WIMStruct *w);

/** 
 * Prints the lookup table of a WIM file.  The lookup table maps SHA1 message
 * digests, as found in the directory entry tree in the WIM file, to file
 * resources in the WIM file.  This table includes one entry for each unique
 * file in the WIM, so it can be quite long.  There is only one lookup table per
 * WIM.
 *
 * @param w 
 * 	The ::WIMStruct for the WIM file.
 *
 * This function has no return value.
 */
void wimlib_print_lookup_table(WIMStruct *w);

/**
 * Prints the metadata of the specified image in a WIM file.  The metadata
 * consists of the security data as well as the directory entry tree, and each
 * image has its own metadata.  
 *
 * @param w 
 * 	The ::WIMStruct for the WIM file.
 * @param image 
 * 	Which image to print the metadata for.  Can be the number of an image,
 * 	or ::WIM_ALL_IMAGES to print the metadata for all images in the WIM.
 *
 * @return 
 * 	@c true on success, @c false on failure.  Note that Wimlib reads the
 * 	metadata into memory only as needed, so this function may fail due to a
 * 	failure to read the metadata.  It also will fail if @a image does not
 * 	specify a valid image(s).
 */
bool wimlib_print_metadata(WIMStruct *w, int image);

/**
 * Prints some basic information about a WIM file.  All information printed by
 * this function is also printed by wimlib_print_header(), but
 * wimlib_print_wim_information() prints some of this information more concisely
 * and in a more readable form.
 *
 * @param w 
 * 	The ::WIMStruct for the WIM file.
 *
 * This function has no return value.  
 */
void wimlib_print_wim_information(const WIMStruct *w);

/**
 * Turns a string specififying the name or number of an image in the WIM into
 * the number of the image.  The images are numbered starting at 1.
 *
 * @param w 
 * 	The ::WIMStruct for the WIM file.
 * @param image_name_or_num  
 * 	A string specifying which image.  If it begins with a number, it is
 * 	taken to be a string specifying the image number.  Otherwise, it is
 * 	taken to be the name of an image, as specified in the XML data for the
 * 	WIM file.  It also may be the keyword "all", which will resolve to
 * 	::WIM_ALL_IMAGES.
 *
 * @return 
 * 	If the string resolved to a single existing image, the number of that
 * 	image, counting starting at 1, is returned.  If the keyword "all" was
 * 	specified, ::WIM_ALL_IMAGES is returned.  Otherwise, ::WIM_NO_IMAGE is
 * 	returned.
 */
int wimlib_resolve_image(WIMStruct *w, const char *image_name_or_num);

/**
 * Sets which image in the WIM is marked as bootable.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param boot_idx
 * 	The number of the image to mark as bootable, or 0 to mark no image as
 * 	bootable.
 * @return @c true on success; @c false if @a boot_idx is not 0 or an existing
 * image.
 */
bool wimlib_set_boot_idx(WIMStruct *w, int boot_idx);

/**
 * Changes the description of an image in the WIM.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param image
 * 	The number of the image for which to change the description.
 * @param description
 * 	The new description to give the image.
 *
 * @return @c true on success; @c false if there is no such image.
 */
bool wimlib_set_image_descripton(WIMStruct *w, int image, 
						const char *description);

/**
 * Changes the name of an image in the WIM.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param image
 * 	The number of the image for which to change the name.
 * @param name
 * 	The new description to give the image.
 *
 * @return @c true on success; @c false if there is no such image.
 */
bool wimlib_set_image_name(WIMStruct *w, int image, const char *name);

/**
 * Sets the link type to use when extracting files from a WIM.  This applies
 * when extracting one image as well as when extracting all images.  Cross-image
 * links may save a lot of space because it is common for files to be referenced
 * multiple times in WIM files.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param link_type
 * 	::WIM_LINK_TYPE_NONE, ::WIM_LINK_TYPE_SYMBOLIC, or ::WIM_LINK_TYPE_HARD.
 *
 * Returns @c true on success, @c false if @a link_type is invalid.
 */

bool wimlib_set_link_type(WIMStruct *w, int link_type);

/**
 * Sets whether WIMLIB is to print error messages when a function fails or not.
 *
 * @param show_messages
 * 	@c true if error messages are to be printed; @c false if error messages
 * 	are not to be printed.
 *
 * This function has no return value.
 */
void wimlib_set_print_errors(bool show_messages);

/**
 * Sets whether Wimlib is to be verbose when extracting files from a WIM or when
 * creating an image from a directory (i.e. whether it will print all affected
 * files or not.)  This is a per-::WIMStruct parameter.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param verbose
 * 	Whether Wimlib is to be verbose when extracting files from the WIM @a w
 * 	using wimlib_extract_image() or when adding an image to @a w using
 * 	wimlib_add_image().
 *
 * This function has no return value.
 */
void wimlib_set_verbose(WIMStruct *w, bool verbose);

/**
 * Sets and creates the directory to which files are to be extracted when
 * extracting files from the WIM.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.
 * @param dir
 * 	The name of the directory to extract files to.
 *
 * @return
 * 	@c true if @a dir was successfully created; @c false if an error
 * 	occurred when creating @a dir.
 */
bool wimlib_set_output_dir(WIMStruct *w, const char *dir);

/**
 * Unmounts a WIM image that was mounted using wimlib_mount().
 *
 * Blocks until it is known whether the mount succeeded or failed.
 *
 * There is currently a design problem with this function because it is hard to
 * know whether the filesystem daemon is still working or whether it has
 * crashed, has been killed, or has reached an infinite loop. However, ideally
 * there should be no infinite loops or crashes in the code,
 * so this wouldn't be much of a problem.  Currently, a timeout of 600 seconds
 * (so long because WIMs can be very large) is implemented so that this function
 * will not wait forever before returning failure.  
 *
 * @param dir
 * 	The directory that the WIM image was mounted on.
 * @param commit
 * 	@c true if changes to the WIM are to be commited.  Ignored for read-only
 * 	mounts.
 * @param check_integrity
 * 	@c true if an integrity table is to be included in the new WIM.  Ignored
 * 	for read-only mounts or if @a commit was @c false.
 *
 * @return
 * 	@c true if the unmount succeeded.  @c false may be returned for any of
 * 	the following reasons:
 * 	- The @b fusermount program, which is needed to perform the unmount,
 * 	could not be found or executed.
 * 	- @b fusermount exited with a failure status when trying to unmount the
 * 	filesystem mounted on @a dir.
 * 	- An error occurred when trying to create or use a POSIX message queue
 * 	to communicate with the daemon servicing the mounted filesystem.
 * 	- @a commit was specified, but daemon servicing the mounted filesystem
 * 	failed to successfully write the new WIM file for one of a number of
 * 	reasons.
 * 	- 600 seconds elapsed, so something probably happened to the filesystem
 * 	daemon.
 */
bool wimlib_unmount(const char *dir, bool commit, bool check_integrity);

/**
 * Writes the WIM to a file.
 *
 * @param w
 * 	The ::WIMStruct for the WIM file.  There may have been in-memory changes
 * 	made to it, which are then reflected in the output file.
 * @param out_wim_path
 * 	The path to the file to write the WIM to.
 * @param image
 * 	The image inside the WIM to write.  Use ::WIM_ALL_IMAGES to include all
 * 	images.
 * @param check_integrity 
 * 	@c true if an integrity table is to be included in the WIM being
 * 	written; false otherwise.
 *
 * @return
 * 	@c true if the WIM was successfully written.  @c false can be returned
 * 	for any of the following reasons:
 * 	- @a out_wim_path could not be opened for writing.
 * 	- An error occurred when trying to write data to the file specified by
 * 	@a out_wim_path.
 * 	- The WIM includes references to files in the outside filesystem, and
 * 	an error occured while trying to read one of these files, perhaps due to
 * 	the file being deleted or changed.
 */
bool wimlib_write(WIMStruct *w, const char *out_wim_path, int image, 
				bool check_integrity);



#endif /* _WIMLIB_H */

